<?php
namespace Teknisa\OPL\Modules\Relatorios\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class CalcularService {
    
    protected $connection;
    protected $newCode;
    
    public function __construct($entityManager, $newCode, $environment, $dataSourceManager) {
        $this->connection = $entityManager->getConnection();
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
    
    public function commit() {
        $this->connection->commit();
    }
    
    public function rollback() {
        $this->connection->rollback();
    }
    
    public function generateCode($newCode) {
        $organizacao = $this->environment->getNrOrgTrab();
        $NRLOGCORTEOL = $this->newCode->getCodeByTableName($newCode, $organizacao);
        return $NRLOGCORTEOL;
    }
    
    public function adqOperadorComprador($param) {
        $sql = "SELECT
                    CDOPERADOR,
                    IDEXSENHCMPOPER,
                    IDCOTCMPOPER,
                    IDAPURCMPOPER,
                    IDAFCMPOPER
                FROM
                    COMPROPER
                WHERE
                        CDCOMPRADOR = :P_CDCOMPRADOR
                    AND CDOPERADOR = :P_CDOPERADOR
                UNION
                SELECT
                    CDOPERADOR,
                    'N'  IDEXSENHCMPOPER,
                    'S'  IDCOTCMPOPER,
                    'S'  IDAPURCMPOPER,
                    'S'  IDAFCMPOPER
                FROM
                    COMPRADOR
                WHERE
                    CDOPERADOR = :P_CDOPERADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function qryOperRelacAux($params) {
        $sql = "SELECT CDOPERADOR, IDEXSENHCMPOPER, IDCOTCMPOPER, IDAPURCMPOPER, IDAFCMPOPER FROM COMPROPER WHERE CDCOMPRADOR = :P_CDCOMPRADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function adqParam($param) {
        $sql = "SELECT PARASUPRI.CDFILIAL, PARASUPRI.IDEXJUSTESCM, PARAMGERAL.IDSISDOS, PARAMGERAL.NRORG, PARASUPRI.IDAPUREFESOLEXT, TRUNC(SYSDATE) AS DATA, PARASUPRI.NRDIAANTENTRAF FROM PARAMGERAL, PARASUPRI, FILIAL WHERE FILIAL.CDFILIAL        = :CDFILCENTCMP AND PARASUPRI.CDFILIAL = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function proceduregetSolicitaCorteOL($row) {
        try {
            $procedure = new StoredProcedure($this->connection, "GERA_SOLICITA_CORTE_OL");
            $procedure->addParam(new Param("P_CDFILIOPELOG", Param::PARAM_INPUT, $row["P_CDFILIOPELOG"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFILLANC", Param::PARAM_INPUT, $row["P_CDFILLANC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDOPERLANC", Param::PARAM_INPUT, $row["P_CDOPERLANC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTINICIAL", Param::PARAM_INPUT, $row["P_DTINICIAL"]["start"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTFINAL", Param::PARAM_INPUT, $row["P_DTFINAL"]["end"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTINICIALAF", Param::PARAM_INPUT, $row["P_DTINICIALAF"]["start"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTFINALAF", Param::PARAM_INPUT, $row["P_DTFINALAF"]["end"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_TIPOCONSULTA", Param::PARAM_INPUT, $row["P_TIPOCONSULTA"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_COMPRADOR", Param::PARAM_INPUT, $row["P_COMPRADOR"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDREGIAO", Param::PARAM_INPUT, $row["P_CDREGIAO"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDPRODINIC", Param::PARAM_INPUT, $row["P_CDPRODINIC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDPRODFIN", Param::PARAM_INPUT, $row["P_CDPRODFIN"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_ROWSAFFECT", Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
            $procedure->addParam(new Param("P_PRXSOLICMP", Param::PARAM_INPUT, $row["P_PRXSOLICMP"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_NRLOGCORTEOL", Param::PARAM_INPUT, $row["P_NRLOGCORTEOL"], Param::PARAM_TYPE_STR));
            return $procedure->execute();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function procedureApuraEfetivacao($row) {
        try {
            $procedure = new StoredProcedure($this->connection, "APURA_EFETIVACAO");
            $procedure->addParam(new Param("P_CDCENTCMP", Param::PARAM_INPUT, $row["P_CDCENTCMP"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_NRSEQAPUR", Param::PARAM_INPUT, $row["P_NRSEQAPUR"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_PRODINI", Param::PARAM_INPUT, $row["P_PRODINI"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_PRODFIN", Param::PARAM_INPUT, $row["P_PRODFIN"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTINI", Param::PARAM_INPUT, $row["P_DTINI"]["start"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTFIN", Param::PARAM_INPUT, $row["P_DTFIN"]["end"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDOPERADOR", Param::PARAM_INPUT, $row["P_CDOPERADOR"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDREGIAO", Param::PARAM_INPUT, $row["P_CDREGIAO"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFILIAL", Param::PARAM_INPUT, $row["P_CDFILIAL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTPROCESSA", Param::PARAM_INPUT, $row["P_DTPROCESSA"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_FILIPROC", Param::PARAM_INPUT, $row["P_FILIPROC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDGRUPFILI", Param::PARAM_INPUT, $row["P_CDGRUPFILI"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_OPTIMIZER_MODE", Param::PARAM_INPUT, $row["P_OPTIMIZER_MODE"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_ROWSAFFECT", Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));
            $procedure->addParam(new Param("P_CDOPERLANC", Param::PARAM_INPUT, $row["P_CDOPERLANC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDCOMPRADOR", Param::PARAM_INPUT, $row["P_CDCOMPRADOR"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_LISTAFILIAL", Param::PARAM_INPUT, $row["P_LISTAFILIAL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_EXTRA", Param::PARAM_INPUT, $row["P_EXTRA"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_NRLOGCORTEOL", Param::PARAM_INPUT, $row["P_NRLOGCORTEOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_IDQTFRAC", Param::PARAM_INPUT, $row["P_IDQTFRAC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_IDUTLITEMRUPTURA", Param::PARAM_INPUT, $row["P_IDUTLITEMRUPTURA"], Param::PARAM_TYPE_STR));
            return $procedure->execute();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getProdIni($param) {
        try{
            $sql = "SELECT MIN(PRODUTO.CDARVPROD) CDARVPROD FROM RELACPRODOL, PRODUTO PRODUTOOL, PRODUTO WHERE RELACPRODOL.CDPRODARVOL = PRODUTOOL.CDPRODUTO AND RELACPRODOL.CDPRARVORIG = PRODUTO.CDPRODUTO AND PRODUTOOL.CDARVPROD BETWEEN :P_CDPRODINIC AND RPAD( :P_CDPRODFIN, 20, 'Z')";
            $stmt = $this->connection->prepare($sql);
            $stmt->execute($param);
            return $stmt->fetchAll();
        } catch(Exception $e){
            throw $e;
        }
    }
    
    public function getProdFin($param) {
        try{
            $sql = "SELECT MAX(PRODUTO.CDARVPROD) CDARVPROD FROM RELACPRODOL, PRODUTO PRODUTOOL, PRODUTO WHERE RELACPRODOL.CDPRODARVOL = PRODUTOOL.CDPRODUTO AND RELACPRODOL.CDPRARVORIG = PRODUTO.CDPRODUTO AND PRODUTOOL.CDARVPROD BETWEEN :P_CDPRODINIC AND RPAD( :P_CDPRODFIN, 20, 'Z')";
            $stmt = $this->connection->prepare($sql);
            $stmt->execute($param);
            return $stmt->fetchAll();
        } catch(Exception $e){
            throw $e;
        }
    }
    
}
